/*
 * Copyright (c) 2015 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <app.h>
#include <Elementary.h>
#include <app_debug.h>
#include <viewmgr.h>

#include "defs.h"
#include "view.h"

SET_TAG(PACKAGE);

struct _appdata {
	const char *name;
	Evas_Object *win;
};

static Evas_Object *_add_win(const char *name)
{
	Evas_Object *win;

	win = elm_win_add(NULL, name, ELM_WIN_BASIC);
	if (!win)
		return NULL;

	elm_win_title_set(win, name);
	elm_win_alpha_set(win, EINA_TRUE);
	evas_object_show(win);

	return win;
}

static bool _create(void *user_data)
{
	struct _appdata *ad;

	if (!user_data) {
		_ERR("Invalid argument");
		return false;
	}

	ad = user_data;

	elm_theme_overlay_add(NULL, THEMEFILE);
	elm_config_focus_move_policy_set(ELM_FOCUS_MOVE_POLICY_CLICK);

	ad->win = _add_win(ad->name);
	if (!ad->win) {
		_ERR("failed to create window");
		return false;
	}

	if (!viewmgr_create(ad->win)) {
		_ERR("failed to create viewmgr");
		evas_object_del(ad->win);
		ad->win = NULL;
		return false;
	}

	viewmgr_add_view(view_home_get_vclass(), NULL);
	viewmgr_add_view(view_recent_get_vclass(), NULL);
	viewmgr_add_view(view_user_get_vclass(), NULL);

	return true;
}

static void _terminate(void *user_data)
{
	struct _appdata *ad;

	if (!user_data)
		return;

	ad = user_data;

	if (ad->win) {
		viewmgr_destroy();
		evas_object_del(ad->win);
	}
}

static void _pause(void *user_data)
{

}

static void _resume(void *user_data)
{

}

static void _control(app_control_h app_control, void *user_data)
{
	struct _appdata *ad;

	if (!user_data) {
		_ERR("Invalid argument");
		return;
	}

	ad = user_data;

	if (ad->win)
		elm_win_activate(ad->win);

	viewmgr_push_view(VIEW_HOME);
}

int main(int argc, char **argv)
{
	struct _appdata ad;

	ui_app_lifecycle_callback_s event_callback = {0,};

	event_callback.create = _create;
	event_callback.terminate = _terminate;
	event_callback.pause = _pause;
	event_callback.resume = _resume;
	event_callback.app_control = _control;

	memset(&ad, 0x00, sizeof(ad));
	ad.name = PACKAGE;

	return ui_app_main(argc, argv, &event_callback, &ad);
}

