/*

 * Copyright (c) 2015 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <Elementary.h>
#include <app_debug.h>
#include <viewmgr.h>
#include <inputmgr.h>

#include "defs.h"
#include "view.h"
#include "data_recent.h"
#include "datamgr.h"
#include "utils.h"

#define MESSAGE_NO_CONTENTS "No Contents"
#define PADDING_BOX 26

enum input_handler_type {
	INPUT_HANDLER_TYPE_EO,
	INPUT_HANDLER_TYPE_DELETE,
	INPUT_HANDLER_TYPE_DISABLE_DELETE
};

enum recent_item_type {
	RECENT_ITEM_ICON = 0x00,
	RECENT_ITEM_PREVIEW = 0x01
};

struct _priv {
	Evas_Object *win;
	Evas_Object *base;
	Evas_Object *del_btn;
	Evas_Object *scr;
	Evas_Object *box;
	Eina_List *list;

	struct _bar_item *cur;
	struct datamgr *dm;
};

struct _bar_item {
	Evas_Object *eo;

	struct datamgr_item *di;
	struct _priv *priv;
};

static void _focused(int id, void *data, Evas_Object *obj,
		Elm_Object_Item *item)
{
	struct _bar_item *bi;

	switch (id) {
	case INPUT_HANDLER_TYPE_EO:
		if (!data)
			return;

		bi = data;
		bi->priv->cur = bi;
		/* fallthrought */
	case INPUT_HANDLER_TYPE_DELETE:
		elm_object_signal_emit(obj, SIG_FOCUS, SRC_PROG);
		break;
	default:
		return;
	}
}

static void _unfocused(int id, void *data, Evas_Object *obj,
		Elm_Object_Item *item)
{
	switch (id) {
	case INPUT_HANDLER_TYPE_EO:
	case INPUT_HANDLER_TYPE_DELETE:
		elm_object_signal_emit(obj, SIG_UNFOCUS, SRC_PROG);
		break;
	default:
		return;
	}
}

static void _key_down(int id, void *data, Evas *e, Evas_Object *obj,
		Evas_Event_Key_Down *ev)
{
	struct _priv *priv;
	struct _bar_item *bi;

	if (!data) {
		_ERR("Invalid argument");
		return;
	}

	switch (id) {
	case INPUT_HANDLER_TYPE_EO:
		bi = data;
		priv = bi->priv;

		if (!strcmp(ev->keyname, KEY_DOWN) ||
				!strcmp(ev->keyname, KEY_BACK) ||
				!strcmp(ev->keyname, KEY_BACK_REMOTE)) {
			elm_object_signal_emit(priv->base, SIG_HIDE, SRC_PROG);
		} else if (!strcmp(ev->keyname, KEY_ENTER) ||
				!strcmp(ev->keyname, KEY_ENTER_REMOTE)) {
			datamgr_select_item(priv->dm, bi->di);
		}
		break;
	case INPUT_HANDLER_TYPE_DELETE:
		priv = data;

		if (!strcmp(ev->keyname, KEY_BACK) ||
				!strcmp(ev->keyname, KEY_BACK_REMOTE)) {
			elm_object_signal_emit(priv->base, SIG_HIDE, SRC_PROG);
		} else if (!strcmp(ev->keyname, KEY_ENTER) ||
				!strcmp(ev->keyname, KEY_ENTER_REMOTE)) {
			datamgr_clear_item(priv->dm);
			elm_object_signal_emit(priv->base, SIG_HIDE, SRC_PROG);
		} else if (!strcmp(ev->keyname, KEY_DOWN)) {
			if (!priv->cur)
				priv->cur = eina_list_data_get(priv->list);
			elm_object_focus_set(priv->cur->eo, EINA_TRUE);
		}
		break;
	case INPUT_HANDLER_TYPE_DISABLE_DELETE:
		priv = data;

		if (!strcmp(ev->keyname, KEY_DOWN) ||
				!strcmp(ev->keyname, KEY_BACK) ||
				!strcmp(ev->keyname, KEY_BACK_REMOTE)) {
			elm_object_signal_emit(priv->base, SIG_HIDE, SRC_PROG);
		}
		break;
	default:
		return;
	}
}

static void _mouse_down(int id, void *data, Evas *e, Evas_Object *obj,
		Evas_Event_Mouse_Down *ev)
{
	struct _bar_item *bi;
	struct _priv *priv;

	if (!data) {
		_ERR("Invalid argument");
		return;
	}

	switch (id) {
	case INPUT_HANDLER_TYPE_EO:
		bi = data;
		priv = bi->priv;

		datamgr_select_item(priv->dm, bi->di);
		break;
	case INPUT_HANDLER_TYPE_DELETE:
		priv = data;

		datamgr_clear_item(priv->dm);
		elm_object_signal_emit(priv->base, SIG_HIDE, SRC_PROG);
		break;
	default:
		return;
	}
}

static void _mouse_move(int id, void *data, Evas *e, Evas_Object *obj,
		Evas_Event_Mouse_Move *ev)
{
	if (!elm_object_focus_get(obj))
		elm_object_focus_set(obj, EINA_TRUE);
}

static input_handler handler = {
	.focused = _focused,
	.unfocused = _unfocused,
	.key_down = _key_down,
	.mouse_down = _mouse_down,
	.mouse_move = _mouse_move
};

static void _hide_done(void *data, Evas_Object *obj, const char *emission,
		const char *source)
{
	viewmgr_pop_view();
}

static void _add_delete_btn(struct _priv *priv, const char *ic_image,
		const char *focus_ic_image, enum input_handler_type type)
{
	Evas_Object *btn, *ic, *focus_ic;

	btn = utils_add_layout(priv->base, GRP_RECENT_DELETE, true,
			PART_RECENT_DELETE_ICON);
	if (!btn) {
		_ERR("failed to add delete icon");
		return;
	}
	evas_object_show(btn);

	ic = utils_add_icon(btn, ic_image, PART_RECENT_DELETE);
	if (!ic) {
		_ERR("failed to add icon");
		evas_object_del(btn);
		return;
	}

	focus_ic = utils_add_icon(btn, focus_ic_image,
			PART_RECENT_DELETE_FOCUS);
	if (!focus_ic) {
		_ERR("failed to add focus icon");
		evas_object_del(btn);
		return;
	}

	inputmgr_add_callback(btn, type, &handler, priv);

	priv->del_btn = btn;
}

static void _show_done(void *data, Evas_Object *obj, const char *emission,
		const char *source)
{
	struct _priv *priv = data;
	struct _bar_item *bi;

	if (!priv->list) {
		elm_object_focus_set(priv->del_btn, EINA_TRUE);
	} else {
		bi = eina_list_data_get(priv->list);
		elm_object_focus_set(bi->eo, EINA_TRUE);
	}
}

static Evas_Object *_create(Evas_Object *win, void *data)
{
	struct _priv *priv;
	struct datamgr *dm;
	Evas_Object *base;

	if (!win) {
		_ERR("Invalid argument");
		return NULL;
	}

	priv = calloc(1, sizeof(*priv));
	if (!priv) {
		_ERR("failed to calloc priv");
		return NULL;
	}

	dm = datamgr_init(datamgr_recent_get_dclass(), VIEW_RECENT);
	if (!dm) {
		_ERR("failed to initialize datamgr");
		free(priv);
		return NULL;
	}

	base = utils_add_layout(win, GRP_RECENT, false, NULL);
	if (!base) {
		_ERR("failed to create base");
		datamgr_fini(dm);
		free(priv);
		return NULL;
	}
	evas_object_size_hint_weight_set(base, EVAS_HINT_EXPAND,
			EVAS_HINT_EXPAND);
	elm_win_resize_object_add(win, base);

	priv->win = win;
	priv->base = base;
	priv->dm = dm;

	viewmgr_set_view_data(VIEW_RECENT, priv);
	elm_object_signal_callback_add(base, SIG_HIDE_DONE, SRC_EDJE,
			_hide_done, NULL);
	elm_object_signal_callback_add(base, SIG_SHOW_DONE, SRC_EDJE,
			_show_done, priv);

	return base;
}

static struct _bar_item *_pack_item(struct _priv *priv, Evas_Object *box,
		struct datamgr_item *di)
{
	Evas_Object *eo, *lbl, *sub_lbl, *thumb, *ic;
	struct _bar_item *bi;
	const char *group;

	switch (di->type) {
	case RECENT_ITEM_PREVIEW:
		group = GRP_RECENT_PREVIEW;
		break;
	case RECENT_ITEM_ICON:
		group = GRP_RECENT_ICON;
		break;
	default:
		return NULL;
	}

	bi = calloc(1, sizeof(*bi));
	if (!bi) {
		_ERR("failed to calloc bar item");
		return NULL;
	}

	eo = utils_add_layout(box, group, true, NULL);
	if (!eo) {
		_ERR("failed to add layout");
		free(bi);
		return NULL;
	}
	evas_object_show(eo);

	ic = utils_add_icon(eo, di->focus_icon, PART_RECENT_ICON);
	if (!ic)
		_ERR("failed to add icon");

	lbl = utils_add_label(eo, di->title, STYLE_LABEL_RECENT_TITLE,
			PART_RECENT_TITLE);
	if (!lbl) {
		_ERR("failed to add title");
		evas_object_del(eo);
		free(bi);
		return NULL;
	}

	if (di->icon) {
		thumb = utils_add_icon(eo, di->icon, PART_RECENT_THUMBNAIL);
		if (!thumb) {
			_ERR("failed to add thumb");
			evas_object_del(eo);
			free(bi);
			return NULL;
		}
	}

	if (di->subtitle) {
		sub_lbl = utils_add_label(eo, di->subtitle,
				STYLE_LABEL_RECENT_SUBTITLE,
				PART_RECENT_SUBTITLE);
		if (!sub_lbl)
			_ERR("failed to add sub title");
	}

	elm_object_focus_next_object_set(eo, priv->del_btn, ELM_FOCUS_UP);
	inputmgr_add_callback(eo, INPUT_HANDLER_TYPE_EO, &handler, bi);
	elm_box_pack_end(box, eo);

	bi->eo = eo;
	bi->di = di;

	return bi;
}

static void _load_recent(struct _priv *priv)
{
	Eina_List *list, *l;
	Evas_Object *scr, *box;
	struct _bar_item *bi;
	struct datamgr_item *di;

	list = datamgr_get_items(priv->dm);
	if (!list) {
		elm_object_part_text_set(priv->base, PART_RECENT_NO_CONTENTS,
				MESSAGE_NO_CONTENTS);
		_add_delete_btn(priv, IMAGE_RECENT_DELETE_DIS,
				IMAGE_RECENT_DELETE_DIS,
				INPUT_HANDLER_TYPE_DISABLE_DELETE);
		return;
	}

	scr = utils_add_scroller(priv->base);
	if (!scr) {
		_ERR("failed to add scroller");
		return;
	}

	box = utils_add_box(scr, true);
	if (!box) {
		_ERR("failed to add box");
		evas_object_del(scr);
		return;
	}
	elm_object_content_set(scr, box);
	elm_object_part_content_set(priv->base, PART_RECENT_CONTENTS, scr);
	elm_object_part_text_set(priv->base, PART_RECENT_NO_CONTENTS, "");
	elm_box_padding_set(box, PADDING_BOX * elm_config_scale_get(), 0);

	_add_delete_btn(priv, IMAGE_RECENT_DELETE_NOR, IMAGE_RECENT_DELETE_FOC,
			INPUT_HANDLER_TYPE_DELETE);

	EINA_LIST_FOREACH(list, l, di) {
		bi = _pack_item(priv, box, di);
		if (!bi)
			continue;

		bi->priv = priv;
		priv->list = eina_list_append(priv->list, bi);
	}
	priv->scr = scr;
	priv->box = box;
}

static void _show(void *data)
{
	struct _priv *priv;

	if (!data) {
		_ERR("Invalid argument");
		return;
	}

	priv = data;

	_load_recent(priv);

	evas_object_show(priv->base);
	elm_object_signal_emit(priv->base, SIG_SHOW, SRC_PROG);
}

static void _unload_recent(struct _priv *priv)
{
	struct _bar_item *bi;

	EINA_LIST_FREE(priv->list, bi) {
		inputmgr_remove_callback(bi->eo, &handler);
		evas_object_del(bi->eo);
		free(bi);
	}

	elm_box_clear(priv->box);
	evas_object_del(priv->scr);
	inputmgr_remove_callback(priv->del_btn, &handler);
	evas_object_del(priv->del_btn);

	priv->del_btn = NULL;
	priv->box = NULL;
	priv->scr = NULL;
	priv->list = NULL;
}

static void _hide(void *data)
{
	struct _priv *priv;

	if (!data) {
		_ERR("Invalid argument");
		return;
	}

	priv = data;

	_unload_recent(priv);

	evas_object_hide(priv->base);
}

static void _destroy(void *data)
{
	struct _priv *priv;

	if (!data) {
		_ERR("Invalid argument");
		return;
	}

	priv = data;

	_unload_recent(priv);

	datamgr_fini(priv->dm);
	evas_object_del(priv->base);
	free(priv);
}

static view_class vclass = {
	.view_id = VIEW_RECENT,
	.create = _create,
	.show = _show,
	.hide = _hide,
	.destroy = _destroy
};

view_class *view_recent_get_vclass(void)
{
	return &vclass;
}

